clc;
clear all;
clf;

%% Definition of parameters
parameters;
L =10;% prediction horizon
N = 6000; 
u=zeros(3,1); 

dt = 1;
iflag = 0;

if iflag==0
    load("closedLoopModel.mat", "A", "B", "D", "K");

end
if iflag==1
    [A, B, D] = linearize_model(u, d, m, dt);
end

%% Defining the size of the input, outputs and state vectors
nx = size(A,1); %Number of states
nu = size(B,2); %Number of inputs
ny = size(D,1); %Number of outputs


%% Defining the Setpoints
sp = zeros(3,N);
sp_ship_vector = zeros(3,N);

sp(1:2,N/4+1:N) = ones(2,3*N/4)*10;
sp(1:2,4001:N)=ones(2,2000)*20;
sp(3,N/3+1:N) = ones(1,2*N/3)*5;
sp(3,4101:N) = ones(1,1900)*10;
%Convert the setpoint to radians to be able to compare it to the yaw state
%of the ship
sp(3,:) = sp(3,:)*pi/180;

%Defining the target endpoint after running the simulation
target_NED = ones(2,N);
target_NED(1:2,1:N) = target_NED(1:2,1:N)*sp(2,N);

%% Defining the disturbances
environment = environmentCondition;
W_c = environment.simulateWater(N,5,'steps');
V_c = zeros(3,N);
% [W_v, gamma] = environment.simulateWind(N,10);
[W_v, gamma] = environment.simulateWind2(N,10);
F_w = zeros(3,N);

%% Calculating Kalman filter gain
if  iflag==0 || iflag==1
% if iflag==1
    % kalman filter gain
    G=0.01*eye(6);
    Q_k = diag([0.09, 0.11, 0.09,0.8,0.5,1.5]);
    R_k = diag([1e3,1e3,1e3]);
    K=dlqe(A,G,D,Q_k,R_k);
end

% Controller weights 
%Weighting parameters, p is for delta u, q is for error
% q = [1e6 0 0; 0 1e7 0; 0 0 1e8];
% p = [1e-5 0 0; 0 1e-6 0; 0 0 1e-10];

q = [1e6 0 0; 0 1e7 0; 0 0 1e8];
p = [1e-4 0 0; 0 1e-4 0; 0 0 1e-10];

% q = [1e6 0 0; 0 1e6 0; 0 0 1e6];
% p = [1e-6 0 0; 0 1e-6 0; 0 0 1e-6];

%[G1, G2, At, Bt, Dt, Rr]=dlqdu_pi(A,B,D,q,p);

%% augmented system matrices for Integral MPC:
%x ̃_(k+1)= A ̃x ̃_k+ B ̃∆u_k  
%y _k= D ̃x ̃_k  
At = [A zeros(nx,ny); D eye(ny,ny)];
Bt= [B ; zeros(ny,nu)];
Dt = [D eye(ny,ny)];
[HdL,OL,OLB]=ss2h(At,Bt,Dt,zeros(ny,nu),L,0); 
FL=[OLB HdL];
Qt=q2qt(q,L);
Rt=q2qt(p,L);
H=FL'*Qt*FL+Rt; % From Theory: H= F^T QF+PH= F^T QF+P



%% Start the model
X=zeros(6,N);
x_est = zeros(6,1);
Y=zeros(3,N);
y_vector = zeros(3,N);
y_NED = zeros(3,N);
U=zeros(3,N); 
xs =zeros(6,1);

start_NED = ones(2,N);
start_NED(1:2,1:N) = start_NED(1:2,1:N).*[y_NED(1,1);y_NED(2,1)];

error = zeros(3,N);
error_NED = zeros(3,N);

x=xs; 

y_old = D*xs; 
x_old=xs;

model = BalchenModel(d,m,x,neu, A_F, A_L, L);

xt = [x-x_old;y_old];
TV=zeros(3,1); %Total value for us
for i=1:N-L
    y = eye(3,6)*x;
    y_vector(:,i) = y;
    R = [cos(x(3)) -sin(x(3)) 0; sin(x(3)) cos(x(3)) 0; 0 0 1];
    y_NED(:,i) = R*y;

    %Calculation of pL
    pL=OL*At*xt;

    %Selection of L next setpoints for comparison
    ref = sp(:,i+1:L+i);
    for k=1:L
        %The selected setpoints need to be converted to the vessel's frame
        ref(:,k)=R'*ref(:,k);
    end
    %Adding just the first reference to the setpoint's vector
    sp_ship_vector(:,i)=ref(:,1);

    ref = ref(:);
    %Finding the value of f
    f=FL'*Qt*(pL-ref);

    %Defining lowe and upper bounds for u
    LB = [-2e5-u(1); -1e6-u(2);-1e6-u(3)];
    UB = [2e5-u(1); 1e6-u(2);1e6-u(3)];
    
    du = quadprog(H,f,[],[],[],[],LB,UB);
    %du=-inv(H)*f;

    
    du = reshape(du,3,L); %arranged control inputs
    u = u + du(:,1);
    TV = TV + abs(du(:,1));

%     if abs(u(1)) > 2e5
%         u(1) = 2e5*sign(u(1));
%     end
%     if abs(u(2)) > 2e5
%         u(2) = 2e5*sign(u(2));
%     end
%     if abs(u(3)) > 1e6
%         u(3) = 1e6*sign(u(3));
% 
%     end


    
    
    x_old = x_est;
    y_old = y;

    U(:,i)=u; 

    F_w(:,i) = model.WindForce(W_v(i),gamma(i), x(3), x(4), x(5), A_F, A_L, L);
    V_c(:,i)= model.CurrentVelocity(W_c(:,i),x);

    %Updating the model, estimation of states and update of xt which is
    %used to calculate pL
    [x, NED_Position] = model.UpdateState(u, W_c(:,i), W_v(i), gamma(i), dt);
    x_est = A*x_est + B * u + K*(y - D*x_est);
    xt = [x_est-x_old;y_old];

    X(:,i)=x; 
    Y(:,i)=y_NED(:,i);

end

%Convert the yaw set point back to degrees for easier interpretation.
sp(3,:) = sp(3,:)*180/pi;
Y(3,:) = Y(3,:)*180/pi;

time = linspace(0,dt*length(U), length(U));

error = rmse(sp',Y')

max_u = max(abs(U'))
mean_u = mean(U')


plot_limit=N-L;

figure(1);
subplot(311);
plot(time(1:plot_limit),Y(1,1:plot_limit), 'r-','LineWidth',1);xlabel('Time [s]');ylabel('Position North of NED[N]');title('Position in North of NED');
hold on;
plot(time(1:plot_limit),sp(1,1:plot_limit), 'b-','LineWidth',1);legend('North Position', 'Set Point North'); grid;
hold off;
subplot(312);
plot(time(1:plot_limit),U(1,1:plot_limit),'g-','LineWidth',1);xlabel('Time [s]');ylabel('Input Signal Surge[N]');legend('Control Signal Surge');grid;title('Input Signal in Surge');
subplot(313);
plot(time(1:plot_limit),F_w(1,1:plot_limit),'c-', 'LineWidth',1);xlabel('Time [s]');ylabel('Wind Force in Surge direction [N]');grid;title('Wind Force in Surge');legend('Wind Force in Surge');

figure(2);
subplot(311);
plot(time(1:plot_limit),Y(2,1:plot_limit), 'r-','LineWidth',1);xlabel('Time [s]');ylabel('Position East of NED[N]');title('Position in Sway');title('Position in East of NED');
hold on;
plot(time(1:plot_limit),sp(2,1:plot_limit), 'b-','LineWidth',1);legend('East Position', 'Set Point East'); grid;
hold off;
subplot(312);
plot(time(1:plot_limit),U(2,1:plot_limit),'g-','LineWidth',1);xlabel('Time [s]');ylabel('Input Signal Sway[N]');legend('Control Signal Sway');grid;title('Input Signal in Sway');
subplot(313);
plot(time(1:plot_limit),F_w(2,1:plot_limit),'c-', 'LineWidth',1);xlabel('Time [s]');ylabel('Wind Force in Sway direction [N]');grid;title('Wind Force in Sway');legend('Wind Force in Sway');

figure(3);
subplot(311);
plot(time(1:plot_limit),Y(3,1:plot_limit), 'r-','LineWidth',1);xlabel('Time [s]');ylabel('Position in Yaw[Deg]');title('Position in Yaw');title('Position in Yaw');
hold on;
plot(time(1:plot_limit),sp(3,1:plot_limit), 'b-','LineWidth',1);legend('Yaw Position', 'Set Point Yaw'); grid;
hold off;
subplot(312);
plot(time(1:plot_limit),U(3,1:plot_limit),'g-','LineWidth',1);xlabel('Time [s]');ylabel('Input Signal Yaw[Nm]');legend('Control Signal Yaw');grid;title('Input Signal in Yaw');
subplot(313);
plot(time(1:plot_limit),F_w(3,1:plot_limit),'c-', 'LineWidth',1);xlabel('Time [s]');ylabel('Wind Moment in Yaw [Nm]');grid;title('Wind Moment');legend('Wind Moment');

figure(4); grid
plot(Y(2,1:N)',Y(1,1:N)', 'c.','LineWidth',1);xlabel('East position [m]'); ylabel('North position [m]');
title('Position in NED coordinate system');
hold on;
plot(target_NED(2,1:N)',target_NED(1,1:N)', 'r*', 'LineWidth',1);
hold off;
hold on;
plot(start_NED(2,1:N)',start_NED(1,1:N)', 'b*', 'LineWidth',1);grid;legend('Position of Vessel in NED Frame', 'Target Position of Vessel in NED Frame', 'Starting Position of Vessell in NED Frame');
hold off;