clc;
clear;

%% Definition of parameters
parameters;

%% The change is calculated every dt seconds
dt=1; 

%% Number of steps
N = 6000;

%% Define signals like input and disturbances for the simulation
x_vector = zeros(6,N);
%x_vector(3,:) = 30*pi/180;
x_NED_vector = zeros(3,N);


%% Initial States
x=x_vector(1:6,1);

%% Input signals and disturbances
u = zeros(3,N);

% u(1,200:400)=10;
% u(2,200:400)=10;

F_w = zeros(3,N);
v = zeros(3,N);
time = linspace(0,dt*length(u), length(u));

%% Defining the disturbances for simulation


environment = environmentCondition;
W_c = environment.simulateWater(N,5,'steps');

%[W_v, gamma] = environment.simulateWind(N,10);
[W_v, gamma] = environment.simulateWind2(N,10);

F_w = zeros(3,N);
V_c = zeros(3,N);

%W_c = zeros(3,N);
% W_c = water_velocity_steps(W_c);
% W_c = water_velocity_sin(W_c);

% W_v = zeros(1,N);
% gamma = zeros(N);
% W_v = slowly(N)*20;% wind speed as a random number
% gamma = wind_direction(N)*360;% wind direction as a random number

%% Using the class now for the PID control
model = BalchenModel(d,m,x,neu, A_F, A_L, L);

sp = zeros(3,N);
sp(1:2,N/4+1:N) = ones(2,3*N/4)*10;
sp(1:2,4001:N)=ones(2,2000)*20;
sp(3,N/3+1:N) = ones(1,2*N/3)*5;
sp(3,4101:N) = ones(1,1900)*10;

target_NED = ones(2,N);
target_NED(1:2,1:N) = target_NED(1:2,1:N)*sp(2,N);



y1 = x(1);
y2 = x(2);
y3 = x(3);

start_NED = ones(2,N);
start_NED(1:2,1:N) = start_NED(1:2,1:N).*[y1;y2];

Kp1 = 3500;
Ti1 = 70;
Td1 = 300;

Kp2 = 5000;
Ti2 = 50;
Td2 = 500;

Kp3 = 10000000;
Ti3 = 100;
Td3 = 500;

% Kp1 = 800;
% Ti1 = 500;
% Td1 = 200;
% 
% Kp2 = 1200;
% Ti2 = 500;
% Td2 = 300;
% 
% Kp3 = 6000000;
% Ti3 = 550;
% Td3 = 145;

% Kp1 = 800;
% Ti1 = 100;
% Td1 = 100;
% 
% Kp2 = 1200;
% Ti2 = 300;
% Td2 = 400;
% 
% Kp3 = 8000;
% Ti3 = 1300;
% Td3 = 200;


surgeController = PIDController(Kp1,Ti1, Td1, dt);
swayController = PIDController(Kp2,Ti2, Td2, dt);
yawController = PIDController(Kp3,Ti3, Td3, dt);

y_vector = zeros(3,N);
NED_Position = 0;
y_NED = zeros(3,N);
TV=zeros(3,1); %Total value for us
du = zeros(3,1);

for i = 1:length(u)
    R = [cos(x(3)) -sin(x(3)) 0; sin(x(3)) cos(x(3)) 0; 0 0 1];
    %y_ship = model.ConvertFromNED(y_vector(:,i));
    y_ship = x(1:3);
    y_vector(:,i) = y_ship;
    y_NED(:,i) = R*y_ship;
    sp_ship = R'*sp(:,i);
    sp_ship(3) = sp_ship(3)*pi/180;

    y1 = y_ship(1);
    y2 = y_ship(2);
    y3 = y_ship(3);

%     u(1,i) = surgeController.ControlSignal(sp(1,i),y1);
%     u(2,i) = swayController.ControlSignal(sp(2,i),y2);
%     u(3,i) = yawController.ControlSignal(sp(3,i),y3);

    u(1,i) = surgeController.ControlSignal(sp_ship(1),y1);
    u(2,i) = swayController.ControlSignal(sp_ship(2),y2);
    u(3,i) = yawController.ControlSignal(sp_ship(3),y3);

    if i>1
        du=u(:,i)-u(:,i-1);
    end
    TV = TV + abs(du); %Finding the Total Value of u to compare the results

    if abs(u(1,i)) > 200000
        u(1,i)= 200000*sign(u(1,i));
    end

    if abs(u(2,i)) > 1e6
        u(2,i)= 1e6*sign(u(2,i));
    end

    if abs(u(3,i)) > 1e6
        u(3,i)= 1e6*sign(u(3,i));
    end

    F_w(:,i) = model.WindForce(W_v(i),gamma(i), x(3), x(4), x(5), A_F, A_L, L);

    [x, NED_Position] = model.UpdateState(u(:,i), W_c(:,i), W_v(i), gamma(i), dt);

end

y_NED(3,:) = y_NED(3,:)*180/pi;

error = rmse(sp',y_NED')

max_u = max(abs(u'))
mean_u = mean(u')


figure(1);grid
subplot(311);
plot(time,y_NED(1,:),'r-', 'LineWidth',1);xlabel('Time[s]');ylabel('Position in North Axis[m]');
title('Position in North of NED');
hold on;
plot(time,sp(1,:),'b-', 'LineWidth',1);grid;legend('Position in North of NED', 'Set Point in North of NED');
hold off;
subplot(312);
plot(time,u(1,:),'g-', 'LineWidth',1);xlabel('Time[s]');ylabel('Control Signal trhust in North Direction [N]');
title('Control Signal for Surge Controller');grid;legend('Control Signal Surge');
subplot(313);
plot(time(1:N),F_w(1,1:N),'Color', [0 0.4470 0.7410],  'LineWidth',1);xlabel('Time [s]');ylabel('Wind Force in Surge direction [N]');grid;title('Wind Force in Surge');legend('Wind Force in Surge');

figure(2);grid
subplot(311);
plot(time,y_NED(2,:),'r-', 'LineWidth',1);xlabel('Time[s]');ylabel('Position in East[m]');
title('Position in East of NED');
hold on;
plot(time,sp(2,:),'b-', 'LineWidth',1);grid;legend('Position in East of NED', 'Set Point in East of NED');
hold off;
subplot(312);
plot(time,u(2,:),'g-', 'LineWidth',1);xlabel('Time[s]');ylabel('Control Signal trhust in Sway Direction [N]');
title('Control Signal for Sway Controller');grid;legend('Control Signal Sway');
subplot(313);
plot(time(1:N),F_w(2,1:N),'Color', [0 0.4470 0.7410],  'LineWidth',1);xlabel('Time [s]');ylabel('Wind Force in Sway direction [N]');grid;title('Wind Force in Sway');legend('Wind Force in Sway');

figure(3);grid
subplot(311);
plot(time,y_NED(3,:),'r-', 'LineWidth',1);xlabel('Time[s]');ylabel('Position in Yaw[deg]');
title('Position in Yaw');
hold on;
plot(time,sp(3,:),'b-', 'LineWidth',1);grid;legend('Position in Yaw', 'Set Point in Yaw');
hold off;
subplot(312);
plot(time,u(3,:),'g-', 'LineWidth',1);xlabel('Time[s]');ylabel('Momment in Yaw [Nm]');
title('Control Signal for Yaw');grid;legend('Control Signal Yaw');
subplot(313)
plot(time(1:N),F_w(3,1:N),'Color', [0 0.4470 0.7410], 'LineWidth',1);xlabel('Time [s]');ylabel('Wind Moment [Nm]');grid;title('Wind Moment');legend('Wind Moment');

figure(4); grid
plot(y_NED(2,1:N)',y_NED(1,1:N)', 'c.','LineWidth',1);xlabel('East position [m]'); ylabel('North position [m]');
title('Position in NED coordinate system');
hold on;
plot(target_NED(2,1:N)',target_NED(1,1:N)', 'r*', 'LineWidth',1);
plot(start_NED(2,1:N)',start_NED(1,1:N)', 'b*', 'LineWidth',1);grid;legend('Position of Vessel in NED Frame', 'Target Position of Vessel in NED Frame', 'Starting Position of Vessell in NED Frame');
hold off;
