clc;
clear all;
clf;

%% Definition of parameters

parameters;
u=zeros(3,1); 

dt = 1;
N = 6000; %The simulation will last 6000 seconds, which is 100 mins.


%% Importing the identified model
iflag = 0;

if iflag==0
    load("closedLoopModel.mat", "A", "B", "D", "K");

end
if iflag==1
    %load("linmodel.mat");
    [A, B, D] = linearize_model(u, d, m, dt);
end


%% kalman filter gain
% The kalman filter is recalculated, as the K matrix from the identified
% model does not perform well during the state estimation.
%if iflag==1
G=0.01*eye(6);
Q_k = diag([0.09, 0.11, 0.09,0.8,0.5,1.5]);
R_k = diag([1e3,1e3,1e3]);
K=dlqe(A,G,D,Q_k,R_k);
%end


%% First, define the disturbances
environment = environmentCondition; %We call an instance of the environment class
W_c = environment.simulateWater(N,5,'steps'); %Simulate the water current
V_c = zeros(3,N);
%[W_v, gamma] = environment.simulateWind(N,10); %Simulate the wind
[W_v, gamma] = environment.simulateWind2(N,10); %Simulate the wind
F_w = zeros(3,N);


%% Defining the setpoints
sp = zeros(3,N);
sp_ship_vector = zeros(3,N);


sp(1:2,N/4+1:N) = ones(2,3*N/4)*10;
sp(1:2,4001:N)=ones(2,2000)*20;
sp(3,N/3+1:N) = ones(1,2*N/3)*5;
sp(3,4101:N) = ones(1,1900)*10;

%Creating a marker fo the target end point only for plotting purposes
target_NED = ones(2,N);
target_NED(1:2,1:N) = target_NED(1:2,1:N)*sp(2,N);

% Controller weights 
%Weighting parameters, p is for delta u, q is for y
% p=1e-6; q=10000; 

q = [1e4 0 0; 0 1e4 0; 0 0 1e6];
p = [1e-5 0 0; 0 1e-5 0; 0 0 1e-9];

%% Obtaining the G matrix
% q = [1e6 0 0; 0 1e7 0; 0 0 1e8];
% p = [1e-5 0 0; 0 1e-6 0; 0 0 1e-10];

[G1, G2, At, Bt, Dt, Rr]=dlqdu_pi(A,B,D,q,p);


%% Initalize the model


%% main control loop
X=zeros(6,N);
x_est = zeros(6,1);
Y=zeros(3,N);
y_vector = zeros(3,N);
y_NED = zeros(3,N);
U=zeros(3,N); 
%R=zeros(3,N); 
xs =zeros(6,1);

%Creating a marker fo the starting point only for plotting purposes
start_NED = ones(2,N);
start_NED(1:2,1:N) = start_NED(1:2,1:N).*[y_NED(1,1);y_NED(2,1)];

%xs(1) = 1;
x=xs; 

y_old = D*xs; 
x_old=xs;

model = BalchenModel(d,m,x,neu, A_F, A_L, L);
TV=zeros(3,1); %U total value

for i=1:N
    y=eye(3,6)*x;

    y_vector(:,i) = y; % Storing the position in the vessel's frame in a vector
    R = [cos(x(3)) -sin(x(3)) 0; sin(x(3)) cos(x(3)) 0; 0 0 1]; % Updating the coordinate conversion matrix
    y_NED(:,i) = R*y; % 

    sp_ship = R'*sp(:,i); % Convert the setpoint from NED to the ship's reference frame.
    sp_ship(3) = sp_ship(3)*pi/180;
    sp_ship_vector(:,i)=sp_ship;


    %Obtaining the updated control signal
    du = G1*(x_est-x_old) + G2*(y_old - sp_ship);
    u = u + du;
    TV = TV + abs(du);

    %Clipping the control signal
    if abs(u(1)) > 2e5
        u(1) = 2e5*sign(u(1));
    end
    if abs(u(2)) > 2e5
        u(2) = 2e5*sign(u(2));
    end
    if abs(u(3)) > 1e6
        u(3) = 1e6*sign(u(3));
    end

    U(:,i)=u; %Storing the control signal for ploting

    x_old = x_est; %Updating the old states for the next iteration
    y_old = y; %Uptading the outputs for the next iteration


    F_w(:,i) = model.WindForce(W_v(i),gamma(i), x(3), x(4), x(5), A_F, A_L, L); %Obtaining the value of the weather disturbance for plotting
    V_c(:,i)= model.CurrentVelocity(W_c(:,i),x); %Calculating the water current force for plotting

    [x, NED_Position] = model.UpdateState(u, W_c(:,i), W_v(i), gamma(i), dt); %Generating the non-linear model's response
    x_est = A*x_est + B * u + K*(y - D*x_est); %Updating the state estimation

    X(:,i)=x; %Storing the model's states for plotting
    Y(:,i)=y_NED(:,i); %Storing the vessel's position for plotting
end

Y(3,:) = Y(3,:)*180/pi; %The vessel's yaw angle needs to be converted to degrees.

time = linspace(0,dt*length(U), length(U));
plot_limit=6000;


error = rmse(sp',Y')
maiximum_u = max(abs(U'))
mean_u = mean(U')

figure(1);
subplot(311);
plot(time(1:plot_limit),Y(1,1:plot_limit), 'r-','LineWidth',1);xlabel('Time [s]');ylabel('Position North of NED[m]');title('Position in North of NED');
hold on;
plot(time(1:plot_limit),sp(1,1:plot_limit), 'b-','LineWidth',1);legend('North Position', 'Set Point North'); grid;
hold off;
subplot(312);
plot(time(1:plot_limit),U(1,1:plot_limit),'g-','LineWidth',1);xlabel('Time [s]');ylabel('Input Signal Surge[N]');legend('Control Signal Surge');grid;title('Input Signal in Surge');
subplot(313);
plot(time(1:plot_limit),F_w(1,1:plot_limit),'c-', 'LineWidth',1);xlabel('Time [s]');ylabel('Wind Force in Surge direction [N]');grid;title('Wind Force in Surge');legend('Wind Force in Surge');

figure(2);
subplot(311);
plot(time(1:plot_limit),Y(2,1:plot_limit), 'r-','LineWidth',1);xlabel('Time [s]');ylabel('Position East of NED[m]');title('Position in Sway');title('Position in East of NED');
hold on;
plot(time(1:plot_limit),sp(2,1:plot_limit), 'b-','LineWidth',1);legend('East Position', 'Set Point East'); grid;
hold off;
subplot(312);
plot(time(1:plot_limit),U(2,1:plot_limit),'g-','LineWidth',1);xlabel('Time [s]');ylabel('Input Signal Sway[N]');legend('Control Signal Sway');grid;title('Input Signal in Sway');
subplot(313);
plot(time(1:plot_limit),F_w(2,1:plot_limit),'c-', 'LineWidth',1);xlabel('Time [s]');ylabel('Wind Force in Sway direction [N]');grid;title('Wind Force in Sway');legend('Wind Force in Sway');

figure(3);
subplot(311);
plot(time(1:plot_limit),Y(3,1:plot_limit), 'r-','LineWidth',1);xlabel('Time [s]');ylabel('Position in Yaw[Deg]');title('Position in Yaw');title('Position in Yaw');
hold on;
plot(time(1:plot_limit),sp(3,1:plot_limit), 'b-','LineWidth',1);legend('Yaw Position', 'Set Point Yaw'); grid;
hold off;
subplot(312);
plot(time(1:plot_limit),U(3,1:plot_limit),'g-','LineWidth',1);xlabel('Time [s]');ylabel('Input Signal Yaw[Nm]');legend('Control Signal Yaw');grid;title('Input Signal in Yaw');
subplot(313);
plot(time(1:plot_limit),F_w(3,1:plot_limit),'c-', 'LineWidth',1);xlabel('Time [s]');ylabel('Wind Moment in Yaw [Nm]');grid;title('Wind Moment');legend('Wind Moment');

figure(4); grid
plot(Y(2,1:N)',Y(1,1:N)', 'c.','LineWidth',1);xlabel('East position [m]'); ylabel('North position [m]');
title('Position in NED coordinate system');
hold on;
plot(target_NED(2,1:N)',target_NED(1,1:N)', 'r*', 'LineWidth',1);
hold off;
hold on;
plot(start_NED(2,1:N)',start_NED(1,1:N)', 'b*', 'LineWidth',1);grid;legend('Position of Vessel in NED Frame', 'Target Position of Vessel in NED Frame', 'Starting Position of Vessell in NED Frame');
hold off;





