classdef environmentCondition 

    methods

        function [W_v, angle] = simulateWind(obj, N,avgSpeed)
            %Function to simulate wind speeds and directions, this then
            %will be used inside the model class to determine wind forces
            %over the ship
            %Enter the number of samples to simulate and an average wind
            %speed around which the wind values will oscillate.
            W_v = zeros (N,1);
            a = rand(4,1);
            for i = 1:N
                if i <= N/4
                    W_v(i,1) = a(1,1);
                elseif i >N/4 && i <= N/2
                    W_v(i,1) = a(2,1);
                elseif i >N/2 && i <= 3*N/4
                    W_v(i,1) = a(3,1);
                else
                    W_v(i,1) = a(4,1);
                end
            end
            W_v = smooth(W_v,0.5)*avgSpeed;
            
            %hrows the wind direction in a fraction of 1, so outside you need to
            %multiply by either 360 or 2*pi depending on whether you want to use
            %degrees or radians.
            angle= zeros(1,N);
            for i = 1:N
                if i <= N/4
                    angle(1,i) = 0;
                elseif i >N/4 && i <= N/2
                    angle(1,i) = pi*1/2;
                elseif i >N/2 && i <= 3*N/4
                    angle(1,i) = pi;
                else
                    angle(1,i) = pi*3/2;
                end
            end
            angle = angle*180/pi;
            angle = smooth(angle,0.5);
        end


        function [W_v, gamma] = simulateWind2(obj, N,avgSpeed)
            W_v = ones (N,1);
            W_v = W_v*avgSpeed;
            middle = round(N/2);
            gamma= ones(1,N);
            gamma(1:middle) = gamma(1:middle)*(45);
            gamma(middle+1:N)= gamma(middle+1:N)*(135);
            gamma = smooth(gamma,0.5);

        end

        function W_c = simulateWater(obj, N, size, typeOfSignal)
            % Use this function to simulate the water currents. Enter the
            % number of samples N and the type of water "signal" for now it
            % should be either 'steps' or 'sins'
            % Enter a size around which the values of water speed will
            % vary.

            W_c = zeros(3,N);

            if strcmp(typeOfSignal,'sins') == true
                for i=1:N
                    W_c(1,i) = sin(pi*i/300)*size;
                    W_c(2,i) = cos(pi*i/300)*size;
                    W_c(3,i) = sin(pi*i/300 + pi/2)*size;
                end
            end

            if strcmp(typeOfSignal, 'steps') == true
                for i = 1:N
                    if i <= N/3
                        W_c(:,i) = [size;0;0];
                    elseif i > N/3 && i <= 2*N/3
                        W_c(:,i) = [0;size;0];
                    elseif i > 2*N/3 && i <= N
                        W_c(:,i) = [0;0;size];
                    end
                end
                W_c(1,:) = smooth(W_c(1,:), 0.5);
                W_c(2,:) = smooth(W_c(2,:), 0.5);
                W_c(3,:) = smooth(W_c(3,:), 0.5);
                
             end

            end

    end
end
