clc;
clear all;
clf;

%% Definition of parameters
parameters;

%% The change is calculated every dt seconds
dt=1; 

%% Number of steps
N = 6000;

%% Define signals like input and disturbances for the simulation
x_vector = zeros(6,N);
%x_vector(3,:) = 30*pi/180;
x_NED_vector = zeros(3,N);
position_NED = zeros(3,1);

Y = zeros(3,N); %Vector Y of outputs where the resulting values will be stored
U2 = zeros(6,N);

%% Initial States
x=x_vector(1:6,1);

%% Input signals and disturbances
u = zeros(3,N);

u(1,500:1000)=100000;
u(1,1500:2000)=200000;
u(1,2500:3000)=250000;
u(1,3500:4000)=250000;
u(1,4000:4500)=200000;
u(1,5000:5500)=150000;

u(2,1:500)=100000;
u(2,1000:1500)=200000;
u(2,2000:2500)=250000;
u(2,3000:3500)=250000;
u(2,4000:4500)=200000;
u(2,5000:5500)=150000;

u(3,1250:1750)=100000;
u(3,2250:2750)=200000;
u(3,3250:3750)=250000;
u(3,3250:3750)=250000;
u(3,4250:4750)=200000;
u(3,5250:5750)=150000;

F_w = zeros(3,N);
v = zeros(3,N);
time = linspace(0,dt*length(u), length(u));

%% Defining the disturbances for simulation


environment = environmentCondition;
W_c = environment.simulateWater(N,0,'steps');

[W_v, gamma] = environment.simulateWind(N,0);

model = BalchenModel(d,m,x,neu, A_F, A_L, L);

% sp = zeros(3,N);
% sp(1:2,N/4+1:N) = ones(2,3*N/4)*40;
% sp(3,N/4+1:N) = ones(1,3*N/4)*10;


%% Impulse Response Experiment Loop
for i = 1:length(u)
    x_vector(:,i)=x;
    R = [cos(x(3)) -sin(x(3)) 0; sin(x(3)) cos(x(3)) 0; 0 0 1];
%     [x, x_NED_vector(:,i)] = model.UpdateState(u(:,i),W_c(:,i), W_v(i), gamma(i), dt);
%     x(1:3) = x_NED_vector(1:3);

    [x, position_NED] = model.UpdateState(u(:,i),W_c(:,i), W_v(i), gamma(i), dt);

    %All of the states with some measurement noise
    y1 = x(1) + x(1)*0.01*(10-rand*(110-90)); 
    y2 = x(2) + x(2)*0.01*(10-rand*(110-90));
    y3 = x(3) + x(3)*0.01*(10-rand*(110-90));


    U2(:,i) = [u(:,i); W_c(i); W_v(i); gamma(i)]; %In this case, the disturbances make part of the inputs

    Y(:,i) = [y1;y2;y3];
       
end

%% Now, uing the DSR toolbox to do system identification
L = 3;
g=0;

%[A,B,D,E,CF,F,x0]=dsr(Y',U',L,g);
prompt = "Select 1 to get the syst. Id info from the .mat file and 2 to get it from the simulation. ";

selection = input(prompt);
plot_limit=6000;

if selection==1
    load("sysIdData.mat");
    [A,B,D,E,K,F,x0,Ef,Yp]=dsr_e(y_vector',U',3,0,2,6);

    figure(2);grid
    subplot(311);
    plot(time(1:plot_limit),U(1,1:plot_limit), 'g');xlabel('Time [s]');ylabel('Input Signal in Surge [N]');
    title('Input in Surge');grid;
    subplot(312);
    plot(time(1:plot_limit),U(2,1:plot_limit), 'g');xlabel('Time [s]');ylabel('Input Signal in Sway [N]');
    title('Input in Sway');grid;
    subplot(313);
    plot(time(1:plot_limit),U(3,1:plot_limit), 'g');xlabel('Time [s]');ylabel('Input Signal in Yaw [Nm]');
    title('Input in Yaw');grid;
    
    figure(3);grid
    subplot(311);
    plot(time(1:plot_limit),y_vector(1,1:plot_limit), 'b');xlabel('Time [s]');ylabel('Output Value in Surge [m]');
    title('Position in Surge');grid;
    subplot(312);
    plot(time(1:plot_limit),y_vector(2,1:plot_limit), 'b');xlabel('Time [s]');ylabel('Output Value in Sway [m]');
    title('Position in Sway');grid;
    subplot(313);
    plot(time(1:plot_limit),y_vector(3,1:plot_limit), 'b');xlabel('Time [s]');ylabel('Output Value in Yaw [degrees]');
    title('Position in Yaw');grid;

else
    %[A,B,D,E,CF,F,x0]=dsr(Y',u',L,g);
    [A,B,D,E,K,F,x0,Ef,Yp]=dsr_e(Y',u',3,0,2,6);

    figure(2);grid
    subplot(311);
    plot(time(1:plot_limit),U(1,1:plot_limit));xlabel('Time');ylabel('Input Signal in Surge [N]');
    title('Input in Surge');
    subplot(312);
    plot(time(1:plot_limit),U(2,1:plot_limit));xlabel('Time');ylabel('Input Signal in Sway [N]');
    title('Input in Sway');
    subplot(313);
    plot(time(1:plot_limit),U(3,1:plot_limit));xlabel('Time');ylabel('Input Signal in Yaw [Nm]');
    title('Input in Yaw');
    
    figure(3);grid
    subplot(311);
    plot(time(1:plot_limit),Y(1,1:plot_limit));xlabel('Time');ylabel('Output Value in North Potision [m]');
    title('Position in North of NED');
    subplot(312);
    plot(time(1:plot_limit),Y(2,1:plot_limit));xlabel('Time');ylabel('Output Value in East Potision [m]');
    title('Position in East of NED');
    subplot(313);
    plot(time(1:plot_limit),Y(3,1:plot_limit));xlabel('Time');ylabel('Output Value in Yaw [degrees]');
    title('Position in Yaw');

end

%% Write Variables File
filename1 = ['closedLoopModel', '.mat'];

%filename2 = ['innovationsModel', '.mat']

save(filename1,"A", "B", "D", "K");

%save(filename2,"A2", "B2", "D2", "K");




%% Plot the System Identification
