clc;
clear;

%% Definition of parameters
parameters;

%% The change is calculated every dt seconds
dt=1; 

%% Number of steps
N = 6000;

%% Define signals like input and disturbances for the simulation
x_vector = zeros(6,N);
%x_vector(3,:) = 30*pi/180;
x_NED_vector = zeros(3,N);

%% Initial States
x=x_vector(1:6,1);

%% Input signals and disturbances
u = zeros(3,N);

% u(1,2000:6000)=10;
% u(2,2000:6000)=10;
% u(3,2000:6000)=10;


F_w = zeros(3,N);
v = zeros(3,N);
time = linspace(0,dt*length(u), length(u));

%% Defining the disturbances for simulation


environment = environmentCondition;
W_c = environment.simulateWater(N,0,'steps');

[W_v, gamma] = environment.simulateWind(N,0);

%% Using the class now for the PID control
model = BalchenModel(d,m,x,neu, A_F, A_L, L);

sp = zeros(3,N);
sp(1:2,N/4+1:N) = ones(2,3*N/4)*50;
sp(3,N/4+1:N) = ones(1,3*N/4)*60;

y1 = x(1);
y2 = x(2);
y3 = x(3);

Kp1 = 800;
Ti1 = 500;
Td1 = 125;

Kp2 = 1200;
Ti2 = 1300;
Td2 = 400;

Kp3 = 8000;
Ti3 = 1500;
Td3 = 500;

surgeController = PIDController(Kp1,Ti1, Td1, dt);
swayController = PIDController(Kp2,Ti2, Td2, dt);
yawController = PIDController(Kp3,Ti3, Td3, dt);

y_vector = zeros(3,N);

for i = 1:length(u)-1

%     [x, y_vector(:,i)] = model.UpdateState(u(:,i-1), W_c(:,i), W_v(i), gamma(i), dt);
    y1 = y_vector(1,i);
    y2 = y_vector(2,i);
    y3 = y_vector(3,i);
%     [x3, x_NED_vector3(:,i)] = model.UpdateState(u(:,i),W_c(:,i), W_v(i), gamma(i), dt);

    u(1,i) = surgeController.ControlSignal(sp(1,i),y1);
    u(2,i) = swayController.ControlSignal(sp(2,i),y2);
    u(3,i) = yawController.ControlSignal(sp(3,i),y3);

    if abs(u(1,i)) > 200000
        u(1,i)= 200000*sign(u(1,i));
    end

    if abs(u(2,i)) > 200000
        u(2,i)= 200000*sign(u(2,i));
    end

    if abs(u(3,i)) > 200000
        u(3,i)= 200000*sign(u(3,i));
    end

    [x, y_vector(:,i+1)] = model.UpdateState(u(:,i), W_c(:,i), W_v(i), gamma(i), dt);

end

figure(1);grid
subplot(211);
plot(time,y_vector(1,:));xlabel('Time');ylabel('Position in Surge[m]');
title('Surge Position');
hold on;
plot(time,sp(1,:));
hold off;
subplot(212);
plot(time,u(1,:));xlabel('Time');ylabel('Control Signal trhust in Surge Direction [N]');
title('Surge Control Signal');

figure(2);grid
subplot(211);
plot(time,y_vector(2,:));xlabel('Time');ylabel('Position in Sway[m]');
title('Sway Position');
hold on;
plot(time,sp(2,:));
hold off;
subplot(212);
plot(time,u(2,:));xlabel('Time');ylabel('Control Signal trhust in Sway Direction [N]');
title('Sway Control Signal');

figure(3);grid
subplot(211);
plot(time,y_vector(3,:));xlabel('Time');ylabel('Position in Yaw[deg]');
title('Yaw');
hold on;
plot(time,sp(3,:));
hold off;
subplot(212);
plot(time,u(3,:));xlabel('Time');ylabel('Control Signal trhust in Surge Direction [N]');
title('Yaw Control Signal');


figure(4); grid
plot(y_vector(2,1:N-2)',y_vector(1,1:N-2)');xlabel('East position [m]'); ylabel('North position [m]');
title('position in NED coordinate system');