function F_w = wind_force(W_v, betta, psi,v_su, v_sw, A_F, A_L, L)
%W_v: wind speed [m/s] in NED frame
%betta: wind direction [deg] in NED frame
%psi: vessel heading [rad]

C_x = 0.6;
C_y = 0.8;
C_n = 0.1;
% C_x, C_y, C_n: wind coefficients: assumed to be constants
betta = betta * pi/180; % convert to radian
u_w = W_v * cos(betta - psi); % wind speed in surge
v_w = W_v * sin (betta - psi);% wind speed in sway
u_rw = v_su - u_w; % wind relative speed in surge
v_rw = v_sw - v_w; % wind relative speed in sway
V_r = sqrt(u_rw^2 + v_rw^2);%wind relative speed
rho = 1.23; % wind density [kg\m^3]
F_w_su = 0.5*rho*C_x*A_F*cos(betta)* V_r^2;
F_w_sw = 0.5*rho*C_y*A_L*sin(betta)* V_r^2;
N_w = 0.5*rho*C_n*A_L*L*sin(2*betta)* V_r^2;
F_w = [F_w_su, F_w_sw, N_w]';
end